#include <stdio.h>

#include <ihx.h>

#include "am29f0x0.h"
#include "flashpgm.h"

using namespace ihx;

#ifndef NULL
# define NULL	0
#endif

am29f0x0::device am29f0x0::devices[] = {
	{"am29f010", 0x01, 0x20,
		{0x4000, 0x4000, 0x4000, 0x4000, 0x4000, 0x4000, 0x4000, 0x4000, 0}},
	{"am29f040", 0x01, 0xA4,
		{0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0x10000, 0}},
	{NULL, 0, 0, {0}}
};

static bool parity(unsigned char d) {
	bool b = false;
	for (int i = 0; i < 8; ++i, d >>= 1)
		b ^= d & 1;
	return b;
}

am29f0x0::am29f0x0(flashpgm *pgm) : memchip(pgm) {
	dev = NULL;
}

void am29f0x0::command(unsigned char cmd, unsigned long cmdaddr) {
	fp->write_at(0x5555, 0xAA);
	fp->write_at(0x2AAA, 0x55);
	fp->write_at(cmdaddr, cmd);
}

int am29f0x0::manufacture_code() {
	unsigned char tmp;
	command(0x90);
	tmp = fp->read_at(0x00);
	reset();
	return parity(tmp) ? tmp : -1;
}

int am29f0x0::device_code() {
	unsigned char tmp;
	command(0x90);
	tmp = fp->read_at(0x01);
	reset();
	return parity(tmp) ? tmp : -1;
}

void am29f0x0::reset() {
	command(0xF0);
}

bool am29f0x0::detect() {
	int mc = manufacture_code(),
	    dc = device_code();
	device *d;

	// traži
	for (d = devices; d->name; ++d) {
		if (d->manufacture == mc && d->code == dc) {
			dev = d;
			return true;
		}
	}

	return false;
}

const char *am29f0x0::device_name() const {
	return dev ? dev->name : "Unknown";
}

unsigned long am29f0x0::space() const {
	if (!dev) return 0;

	unsigned long *l = dev->sector_size;
	unsigned long size = 0;
	while (*l)
		size += *(l++);
	return size;
}

unsigned long am29f0x0::sector_address(int id) const {
	if (!dev) return -1U;

	unsigned long *l = dev->sector_size;
	unsigned long a = 0;
	while (id-- && *l)
		a += *(l++);
	return (id != -1 || !*l) ? -1U : a;
}

bool am29f0x0::toggle_check(unsigned char b1, unsigned char b2) {
	return !!((b1 ^ b2) & 0x40);
}

bool am29f0x0::poll(unsigned long addr) {
	unsigned char b1, b2;
	b2 = fp->read_at(addr);
	do {
		b1 = b2;
		b2 = fp->read_at(addr);
		if (!toggle_check(b1, b2))
			break;
	} while (!(b2 & 0x20));	// !DQ5 = !time_limit
	b1 = b2;
	b2 = fp->read_at(addr);
	return !toggle_check(b1, b2);
}

bool am29f0x0::erase_all() {
	bool ok;
	init_progress(PRGT_ERASE, "Erasing whole memory...");
	command(0x80);
	command(0x10);
	ok = poll(0);
	reset();
	done_progress(ok);
	return ok;
}

bool am29f0x0::erase_sector(unsigned long sectaddr) {
	bool ok;
	init_progress(PRGT_ERASE, "Erasing sector...");
	command(0x80);
	command(0x30, sectaddr);
	ok = poll(sectaddr);
	reset();
	done_progress(ok);
	return ok;
}

bool am29f0x0::write_byte(unsigned long addr, unsigned char d) {
	command(0xA0);
	fp->write_at(addr, d);
	return poll(addr);
}

bool am29f0x0::write(const Buffer &buf) {
	if (buf.LastAddress() >= space()) {
		fprintf(stderr, "Data will not fit to RAM!\n");
		return false;
	}

	unsigned long size = buf.CalcSize(), dsize = 0;
	bool ok = true;

	init_progress(PRGT_WRITE, "Writing 0x%lX bytes...", size);
	for (Buffer::const_iterator i = buf.begin(); i != buf.end(); ++i) {
		const unsigned char *mem = i->GetBuffer();
		for (unsigned int l = 0; l < i->GetSize(); ++l) {
			ok = write_byte(i->address + l, mem[l]);
			if (!ok) goto kraj;
			show_progress(++dsize * 100 / size);
		}
		info_progress("Element 0x%X, 0x%X bytes done!", i->address, i->GetSize());
	}
kraj:
	done_progress(ok);
	reset();
	return ok;
}
