#include <err.h>

#include "flashpgm.h"
#include "lpt.h"

#ifndef NULL
# define NULL	(0)
#endif

flashpgm::flashpgm(lpt *ppd) {
	if (!ppd) {
		deleteppd = true;
		pp = new lpt();
		if (!pp->init()) {
			delete pp;
			pp = NULL;
		}
	} else {
		deleteppd = false;
		pp = ppd;
	}

	for (unsigned int i = 0; i < N_REGS; ++i)
		c_reg[i] = -1;
}

flashpgm::~flashpgm() {
	if (deleteppd && pp)
		delete pp;
}

bool flashpgm::preinit() {
	if (!pp)
		return false;

	pp->mode(lptm_standard);
	pp->output_enable(true);

	pp->init_printer(false);	// 1=force 137 OE
	pp->select_printer(false);	// 137=strober
	pp->auto_linefeed(false);	// 1=data oe to flash

	return true;
}

void flashpgm::init() {
	// a16@0, a17@1, a18, ce, oe, we
	reg(2, 0x38);	// ce, oe, we = 1
	led(true, false);
}

bool flashpgm::_detect() {
	bool tmp;

	pp->init_printer(true);

	select(7);
	tmp = pp->ack();
	select(6);
	tmp ^= pp->ack();

	pp->init_printer(false);

	return tmp;
}

bool flashpgm::detect(int n) {
	while (n--)
		if (_detect())
			return true;
	return false;
}

void flashpgm::select(int id) {
	if ((unsigned int)id >= N_REGS)
		errx(1, "Internal error: selecting nonexistent register %d!", id);
	if (c_select == id && id != 7)
		return;

	pp->select_printer(true);	// 1=137, 0=strober
	write(c_select = id);
	pp->select_printer(false);
}

unsigned char flashpgm::read() {
	unsigned char tmp;
	select(4);
	pp->output_enable(false);
	pp->strobe(true);
	tmp = pp->data();
	pp->strobe(false);
	pp->output_enable(true);
	return tmp;
}

void flashpgm::reg(int id, unsigned char data) {
	if ((unsigned int)id >= N_REGS)
		errx(1, "Internal error: writing to nonexistent register %d!", id);
	if (data == c_reg[id]) {
		return;
}
	select(id);
	write(c_reg[id] = data);
}

void flashpgm::write(unsigned char data) {
	pp->data(data);
	if (c_select == 3)
		pp->strobe_pulse(1);
	else
		pp->strobe_pulse();
}

void flashpgm::led(bool green, bool red) {
	reg(5, green | (red << 1));
}

void flashpgm::address(unsigned long addr) {
	reg(2, (reg(2) & 0xF8) | ((addr >> 16) & 0x07));
	reg(1, (addr >> 8) & 0xFF);
	reg(0, (addr >> 0) & 0xFF);
}

void flashpgm::write_at(unsigned long addr, unsigned char data) {
	address(addr);
	reg(3, data);
	pp->auto_linefeed(true);
	flash_ctrl(true, false, true);
	flash_ctrl(false, false, false);
	pp->auto_linefeed(false);
}

unsigned char flashpgm::read_at(unsigned long addr) {
	unsigned char tmp;
	address(addr);
	flash_ctrl(true, true, false);
	tmp = read();
	flash_ctrl(false, false, false);
	return tmp;
}

void flashpgm::flash_ctrl(bool ce, bool oe, bool we) {
	unsigned char d = (!ce << 3) | (!oe << 4) | (!we << 5);
	reg(2, (reg(2) & 0xC7) | d);
}
