#ifndef _MEMCHIP_H_
#define _MEMCHIP_H_

#include <stdarg.h>

#include <ihx.h>

#include "flashpgm.h"

using namespace ihx;

class memchip {
	public:
		typedef enum {
			PRGT_DONE,
			PRGT_ERASE,
			PRGT_WRITE,
			PRGT_READ,
		} prgtype;

		memchip(flashpgm *prog) : fp(prog) {}
		virtual ~memchip() {}
		virtual unsigned long space() const = 0;
		virtual const char *device_name() const = 0;
		virtual unsigned long sector_address(int id) const;
		virtual bool erase_sector(unsigned long sect_addr) {
			return erase_all();
		}
		virtual bool erase_all();

		void init_progress(prgtype type, const char *s, ...);
		void show_progress(int percent);
		void info_progress(const char *str, ...);
		void done_progress(bool ok = true);

		virtual BuffElem read(unsigned long from = 0, unsigned long size = -1U);
		virtual bool write(const Buffer &buf);
		virtual bool verify(const Buffer &buf);

	protected:
		flashpgm *fp;

	private:
		int lper, ltime;
		prgtype ltype;
		bool lleds;

		void leds_progress(prgtype type, bool on = true);
		static int msec_time();
};

#endif
